<?php
/*
Plugin Name: WordPress Security Lab
Plugin URI: http://www.wordpresssecuritylab.com/
Description: Check your site for <strong>security vulnerabilities</strong> and get precise suggestions for corrective actions on passwords, user accounts, file permissions, database security, version hiding, plugins, themes and other security aspects.
Author: eConsultants.it Ltd
Version: 1.0
Author URI: http://econsultants.it/
*/


if (!function_exists('add_action')) {
  die('Please don\'t open this file directly!');
}

define('WPSL_VER', '1.0');
define('WPSL_DIC', plugin_dir_path(__FILE__) . 'wpsl-dictionary.txt');
define('WPSL_OPTIONS_KEY', 'wpsl_results');
define('WPSL_INSTALL_TIME', 'wpsl_install_time');
define('WPSL_MAX_USERS_ATTACK', 5);
define('WPSL_MAX_EXEC_SEC', 200);
define('WPSL_FIXIT_KEY', 'wpsl_fixit');
define('WPSL_STORE_URL', 'http://www.wordpresssecuritylab.com');
define('WPSL_ITEM_NAME', 'WP Security Lab Plugin Download');

if( !class_exists( 'EDD_SL_Plugin_Updater' ) ) {
  include( dirname( __FILE__ ) . '/EDD_SL_Plugin_Updater.php' );
}

$license_key = trim( get_option( 'wpsl_license_key' ) );
$edd_updater = new EDD_SL_Plugin_Updater(WPSL_STORE_URL, __FILE__, array(
    'version'   => WPSL_VER,
    'license'   => $license_key,
    'item_name' => WPSL_ITEM_NAME,
    'author'   => 'eConsultants.it Ltd'
  )
);

require_once 'wpsl-tests.php';
require_once 'wpsl-core-scanner.php';
require_once 'wpsl-captcha.php';

class wpsl {
  // init plugin
  function init() {
    // does the user have enough privilages to use the plugin?
    if (is_admin() && current_user_can('administrator')) {
      // this plugin requires WP v3.1
      if (!version_compare(get_bloginfo('version'), '3.3',  '>=')) {
        add_action('admin_notices', array(__CLASS__, 'min_version_error'));
        return;
      } else {
        // add menu item to tools
        add_action('admin_menu', array(__CLASS__, 'admin_menu'));

        // aditional links in plugin description
        add_filter('plugin_action_links_' . basename(dirname(__FILE__)) . '/' . basename(__FILE__),
                   array(__CLASS__, 'plugin_action_links'));
        add_filter('plugin_row_meta', array(__CLASS__, 'plugin_meta_links'), 10, 2);

        // enqueue scripts
        add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_scripts'));

        // register ajax endpoints
        add_action('wp_ajax_wpsl_run_tests', array(__CLASS__, 'run_tests'));
        add_action('wp_ajax_wpsl_helpme', array(__CLASS__, 'create_user'));
        add_action('wp_ajax_wpsl_fixit', array(__CLASS__, 'fixit'));

        // warn if tests were not run
        add_action('admin_notices', array(__CLASS__, 'run_tests_warning'));

        // warn if Wordfence is active
        add_action('admin_notices', array(__CLASS__, 'wordfence_warning'));
      } // if version

      add_action('admin_init', array(__CLASS__, 'edd_activate_license'));
      add_action('admin_init', array(__CLASS__, 'edd_deactivate_license'));
    } // if

    self::apply_fixit();
  } // init


  // add links to plugin's description in plugins table
  function plugin_meta_links($links, $file) {
    $support_link = '<a target="_blank" href="mailto:support@wordpresssecuritylab.com" title="Contact support">Support</a>';

    if ($file == plugin_basename(__FILE__)) {
      $links[] = $support_link;
    }

    return $links;
  } // plugin_meta_links


  // add settings link to plugins page
  function plugin_action_links($links) {
    $settings_link = '<a href="tools.php?page=wpsl" title="WP Security Lab">Analyze site</a>';
    array_unshift($links, $settings_link);

    return $links;
  } // plugin_action_links


  // test if plugin's page is visible
  function is_plugin_page() {
    $current_screen = get_current_screen();

    if ($current_screen->id == 'tools_page_wpsl') {
      return true;
    } else {
      return false;
    }
  } // is_plugin_page

  // enqueue CSS and JS scripts on plugin's pages
  function enqueue_scripts() {
    if (self::is_plugin_page()) {
      $plugin_url = plugin_dir_url(__FILE__);

      wp_enqueue_style('wp-jquery-ui-dialog');
      wp_enqueue_script('jquery-ui-dialog');
      wp_enqueue_script('jquery-ui-tabs');
      wp_enqueue_script('wpsl-jquery-plugins', $plugin_url . 'js/wpsl-jquery-plugins.js', array(), WPSL_VER, true);
      wp_enqueue_script('wpsl-js', $plugin_url . 'js/wpsl-common.js', array(), WPSL_VER, true);
      wp_enqueue_script('wpsl-core', $plugin_url . 'js/wpsl-core.js', array(), WPSL_VER, true);
      wp_enqueue_script('wpsl-snippet', $plugin_url . 'js/snippet.min.js', array(), WPSL_VER, true);

      wp_enqueue_style('wpsl-css', $plugin_url . 'css/wpsl-style.css', array(), WPSL_VER);
      wp_enqueue_style('wpsl-snippet', $plugin_url . 'css/snippet.min.css', array(), WPSL_VER);
    } // if
  } // enqueue_scripts


  // add entry to admin menu
  function admin_menu() {
    add_management_page('WP Security Lab', 'WP Security Lab', 'manage_options', 'wpsl', array(__CLASS__, 'options_page'));
  } // admin_menu


  // display warning if test were never run
  function run_tests_warning() {
    $tests = get_option(WPSL_OPTIONS_KEY);

    if (self::is_plugin_page() && !$tests['last_run']) {
      echo '<div id="message" class="error"><p>WordPress Security Lab <strong>tests were never run.</strong> Click "Run tests" to run them now and analyze your site for security vulnerabilities.</p></div>';
    } elseif (self::is_plugin_page() && (current_time('timestamp') - 30*24*60*60) > $tests['last_run']) {
      echo '<div id="message" class="error"><p>WordPress Security Lab <strong>tests were not run for more than 30 days.</strong> It\'s advisable to run them once in a while. Click "Run tests" to run them now and analyze your site for security vulnerabilities.</p></div>';
    }
  } // run_tests_warning


  // display warning if Wordfence plugin is active
  function wordfence_warning() {
    if (defined('WORDFENCE_VERSION') && WORDFENCE_VERSION) {
      echo '<div id="message" class="error"><p>Please <strong>deactivate Wordfence plugin</strong> before running WordPress Security Lab tests. Some tests are detected as site attacks by Wordfence and hence can\'t be performed properly. Activate Wordfence once you\'re done testing.</p></div>';
    }
  } // wordfence_warning


  // display warning if test were never run
  function min_version_error() {
    echo '<div id="message" class="error"><p>WordPress Security Lab <b>requires WordPress version 3.3</b> or higher to function properly. You\'re using WordPress version ' . get_bloginfo('version') . '. Please <a href="' . admin_url('update-core.php') . '" title="Update WP core">update</a>.</p></div>';
  } // min_version_error

  // whole options page
  function options_page() {
    // does the user have enough privilages to access this page?
    if (!current_user_can('administrator'))  {
      wp_die('You do not have sufficient permissions to access this page.');
    }

    $tabs = array();
    if (self::license()) {
      $tabs[] = array('id' => 'sn_tests', 'class' => '', 'label' => 'Tests', 'callback' => array('self', 'tests_table'));
      $tabs[] = array('id' => 'sn_help', 'class' => 'sn_help', 'label' => 'Test details, tips &amp; help', 'callback' => array('self', 'help_table'));
      $tabs = apply_filters('sn_tabs', $tabs);
    }

    $tabs[] = array('id' => 'sn_license', 'class' => 'sn_license', 'label' => 'License', 'callback' => array('self', 'license_page'));

    echo '<div class="wrap">' . get_screen_icon('sn-lock');
    echo '<h2>WordPress Security Lab</h2>';

    echo '<div id="tabs">';
    echo '<ul>';
    foreach ($tabs as $tab) {
      echo '<li><a href="#' . $tab['id'] . '" class="' . $tab['class'] . '">' . $tab['label'] . '</a></li>';
    }
    echo '</ul>';

    foreach ($tabs as $tab) {
      echo '<div id="' . $tab['id'] . '">';
      call_user_func($tab['callback']);
      echo '</div>';
    }

    echo '</div>'; // tabs
    echo '</div>'; // wrap
  } // options_page


  // display tests help & info
  function help_table() {
    if (!self::license()) {
      echo 'Cheating are you?';
      return;
    }

    require_once 'wpsl-tests-description.php';
  } // help_table


  // display tests table
  function tests_table() {
    if (!self::license()) {
      echo 'Cheating are you?';
      return;
    }

    // get test results from cache
    $tests = get_option(WPSL_OPTIONS_KEY);

    echo '<p class="submit"><input type="submit" value=" Run tests " id="run-tests" class="button-primary" name="Submit" />&nbsp;&nbsp;';

    if ($tests['last_run']) {
      echo '<span class="sn-notice">Tests were last run on: ' . date(get_option('date_format') . ' ' . get_option('time_format'), $tests['last_run']) . '.</span>';
    }

    echo '</p>';

    echo '<p><strong>Please read!</strong> These tests only serve as suggestions! Although they cover years of best practices getting all test <i>green</i> will not guarantee your site will not get hacked. Likewise, getting them all <i>red</i> doesn\'t mean you\'ll certainly get hacked. Please read each test\'s detailed information to see if it represents a real security issue for your site. Suggestions and test results apply to public, production sites, not local, development ones. <br /> If you need an in-depth security analysis please hire a security expert. <button id="helpme" class="button">Help me!</button></p>';
    echo '<p>The first 14 tests, when secured, will protect this site from all known daily hacks and infections.<br>
The following 19 tests, when secured, will develop a much more hardened security for this site.</p><br />';

    if ($tests['last_run']) {
      echo '<table class="wp-list-table widefat" cellspacing="0" id="security-ninja">';
      echo '<thead><tr>';
      echo '<th class="sn-status">Status</th>';
      echo '<th>Test description</th>';
      echo '<th>Test results</th>';
      echo '<th>&nbsp;</th>';
      echo '</tr></thead>';
      echo '<tbody>';

      if (is_array($tests['test'])) {
        // test Results
        foreach($tests['test'] as $test_name => $details) {
          echo '<tr>
                  <td class="sn-status">' . self::status($details['status']) . '</td>
                  <td>' . $details['title'] . '</td>
                  <td>' . $details['msg'] . '</td>
                  <td class="sn-details">';
          if (!$details['status'] && isset(wpsl_tests::$security_tests[$test_name]['fixit']) && wpsl_tests::$security_tests[$test_name]['fixit']) {
            echo ' <a href="#" data-test="' . $test_name . '" class="button action fixit">Fix it!</a>';
          }
          echo '<a href="#' . $test_name . '" class="button action details">Details, tips &amp; help</a> ';
          echo '</td>
                </tr>';
        } // foreach ($tests)
      } else { // no test results
        echo '<tr>
                <td colspan="4">No test results are available. Click "Run tests" to run tests now.</td>
              </tr>';
      } // if tests

      echo '</tbody>';
      echo '<tfoot><tr>';
      echo '<th class="sn-status">Status</th>';
      echo '<th>Test description</th>';
      echo '<th>Test results</th>';
      echo '<th>&nbsp;</th>';
      echo '</tr></tfoot>';
      echo '</table>';
    } // if $results

    echo '<div id="helpme-dialog">A new user with the administrator\'s privileges will be created and the username and password sent to support@worspresssecuritylab.com in order for them to be able to login and diagnose the issue you\'re having.<br>
Issue details:<br>
     <textarea id="helpme-details" rows="5"></textarea>

     <br><br><strong>Click to send for assistance</strong><br><a id="helpme-do" href="#" class="button secondary">I understand, I need assistance</a></div>';
  } // tests_table

  // create user for help function
  function create_user() {
    $tmp = rand(1, 1000);
    $username = 'support-' . $tmp;
    $useremail = 'support@wordpresssecuritylab.com';
    $userid = username_exists($username);
    if (!$userid && email_exists($useremail) == false ) {
      $password = wp_generate_password(12);
      $userid = wp_create_user($username, $password, $useremail );
      echo $userid;
      if ($userid) {
        $user = new WP_User($userid);
        $user->set_role('administrator');
      } else {
        die('0');
      }

      global $current_user;
      get_currentuserinfo();

      $email = get_bloginfo('wpurl') . '/wp-admin/' . "\n";
      $email .= 'user\'s email: ' . $current_user->user_email . "\n";
      $email .= 'username: ' . $username . "\n";
      $email .= 'password: ' . $password . "\n";
      $email .= 'problem: ' . $_POST['details'] . "\n";
      wp_mail($useremail, 'Support request via WP Security Lab', $email);
      die($email);
    } else {
      die('0');
    }
  } // create_user


  // run all tests; via AJAX
  function run_tests($return = false) {
    @set_time_limit(WPSL_EXEC_SEC);
    $test_count = 0;
    $test_description = array('last_run' => current_time('timestamp'));

    foreach(wpsl_tests::$security_tests as $test_name => $test){
      if ($test_name[0] == '_') {
        continue;
      }
      $response = wpsl_tests::$test_name();

      $test_description['test'][$test_name]['title'] = $test['title'];
      $test_description['test'][$test_name]['status'] = $response['status'];

      if (!isset($response['msg'])) {
        $response['msg'] = '';
      }

      if ($response['status'] == 10) {
        $test_description['test'][$test_name]['msg'] = sprintf($test['msg_ok'], $response['msg']);
      } elseif ($response['status'] == 0) {
        $test_description['test'][$test_name]['msg'] = sprintf($test['msg_bad'], $response['msg']);
      } else {
        $test_description['test'][$test_name]['msg'] = sprintf($test['msg_warning'], $response['msg']);
      }
      $test_count++;
    } // foreach

    if ($return) {
      return $test_description;
    } else {
      update_option(WPSL_OPTIONS_KEY, $test_description);
      die('1');
    }
  } // run_test


  // convert status integer to button
  function status($int) {
    if ($int == 0) {
      $string = '<span class="sn-error">Not secure</span>';
    } elseif ($int == 10) {
      $string = '<span class="sn-success">Secure</span>';
    } else {
      $string = '<span class="sn-warning">Warning</span>';
    }

    return $string;
  } // status

  function remove_version() {
    return '';
  }
  function wrong_login($msg) {
    if (stripos($msg, 'invalid username') || stripos($msg, 'The password you entered')) {
      return '<strong>ERROR:</strong> Wrong username or password.';
    }
    
    return $msg;
  }

  function apply_fixit() {
    $options = get_option(WPSL_FIXIT_KEY);

    if (isset($options['captcha']) && $options['captcha']) {
      add_action('login_form', array(__CLASS__, 'captcha_print'));
      add_filter('login_errors', array(__CLASS__, 'login_captcha_errors'), 3);
      add_filter('wp_authenticate_user', array(__CLASS__, 'authenticate_user_check'), 15, 2);
    }
    
    if (isset($options['wp_header_meta']) && $options['wp_header_meta']) {
      add_filter('the_generator', array(__CLASS__, 'remove_version'), 99);
    }

    if (isset($options['check_failed_login_info']) && $options['check_failed_login_info']) {
      add_filter('login_errors', array(__CLASS__, 'wrong_login'), 99);
    }

    if (isset($options['wlw_meta']) && $options['wlw_meta']) {
      remove_action('wp_head', 'wlwmanifest_link');
    }

    if (isset($options['rpc_meta']) && $options['rpc_meta']) {
      remove_action('wp_head', 'rsd_link');
    }

    if (isset($options['file_editor']) && $options['file_editor']) {
      define('DISALLOW_FILE_EDIT', true);
    }

    if (isset($options['debug_check']) && $options['debug_check']) {
      define('WP_DEBUG', false);
    }
  } // apply fixit

  // registration captcha errors
  function login_captcha_errors($errors) {
    if (get_transient('wpsl_captcha_failed')) {
      delete_transient('wpsl_captcha_failed');
      $errors = '<strong>ERROR:</strong> Wrong captcha value!';
    }

    return $errors;
  } // register_captcha_errors

  // check captcha code
  function authenticate_user_check($user, $password) {
    if (isset($_POST['wpsl_captcha']) &&
        $_POST['wpsl_captcha'] != wpsl_captcha::get()) {
      set_transient('wpsl_captcha_failed', 5);
      $user = null;
    }

    return $user;
  } // authenticate_user_check

  function captcha_print() {
    echo '<p><label for="wpsl_captcha">Are you human? Please solve: ';
    echo '<img src="' . plugins_url('wpsl-captcha.php?generate-image=true&noise=0&rnd=' . rand(0, 10000), __FILE__) . '" alt="Captcha" title="Captcha" />';
    echo '<br /><input class="input" type="text" name="wpsl_captcha" id="wpsl_captcha" />';
    echo '</label></p><br />';
  } // captcha_print

  function fixit() {
    $options = get_option(WPSL_FIXIT_KEY);

    switch ($_POST['test']) {
      case 'anyone_can_register':
        update_option('users_can_register', 0);
        die('10');
      break;
      case 'readme_check':
        $tmp = @rename(ABSPATH . 'readme.html', ABSPATH . '_readme.html');
        if ($tmp) {
          die('10');
        } else {
          die('0');
        }
      break;
      case 'wp_header_meta':
        $options['wp_header_meta'] = true;
        update_option(WPSL_FIXIT_KEY, $options);
        die('10');
      break;
      case 'check_failed_login_info':
        $options['check_failed_login_info'] = true;
        update_option(WPSL_FIXIT_KEY, $options);
        die('10');
      break;
      case 'wlw_meta':
        $options['wlw_meta'] = true;
        update_option(WPSL_FIXIT_KEY, $options);
        die('10');
      break;
      case 'rpc_meta':
        $options['rpc_meta'] = true;
        update_option(WPSL_FIXIT_KEY, $options);
        die('10');
      break;
      case 'file_editor':
        $options['file_editor'] = true;
        update_option(WPSL_FIXIT_KEY, $options);
        die('10');
      break;
      case 'debug_check':
        $options['debug_check'] = true;
        update_option(WPSL_FIXIT_KEY, $options);
        die('10');
      break;
      case 'captcha':
        $options['captcha'] = true;
        update_option(WPSL_FIXIT_KEY, $options);
        die('10');
      break;
      case 'install_file_check':
        $tmp = @rename(ABSPATH . 'wp-admin/install.php', ABSPATH . 'wp-admin/_install.php');
        if ($tmp) {
          die('10');
        } else {
          die('0');
        }
      break;
      case 'upgrade_file_check':
        $tmp = @rename(ABSPATH . 'wp-admin/upgrade.php', ABSPATH . 'wp-admin/_upgrade.php');
        if ($tmp) {
          die('10');
        } else {
          die('0');
        }
      break;
      default:
      break;
    }

    die('0');
  } // fixit

  // clean-up when deactivated
  function deactivate() {
    delete_option(WPSL_OPTIONS_KEY);
    delete_option(WPSL_FIXIT_KEY);
    delete_option(WPSL_CS_OPTIONS_KEY);
    delete_option('wpsl_license_key');
    delete_option('wpsl_license_status');
  } // deactivate


function edd_sanitize_license( $new ) {
  $old = get_option( 'wpsl_license_key' );
  if( $old && $old != $new ) {
    delete_option( 'wpsl_license_status' ); // new license has been entered, so must reactivate
  }
  return $new;
}

  function edd_deactivate_license() {
	// listen for our activate button to be clicked
	if( isset( $_POST['_edd_license_deactivate'] ) ) {

		// run a quick security check 
	 	//if( ! check_admin_referer( 'edd_nonce', 'edd_nonce' ) ) 	
		//	return; // get out if we didn't click the Activate button

		// retrieve the license from the database
		$license = trim( get_option( 'wpsl_license_key' ) );
			
		// data to send in our API request
		$api_params = array( 
			'edd_action'=> 'deactivate_license', 
			'license' 	=> $license, 
			'item_name' => urlencode( WPSL_ITEM_NAME ) // the name of our product in EDD
		);
		
		// Call the custom API.
		$response = wp_remote_get( add_query_arg( $api_params, WPSL_STORE_URL ), array( 'timeout' => 15, 'sslverify' => false ) );

		// make sure the response came back okay
		if ( is_wp_error( $response ) )
			return false;

		// decode the license data
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );
		
		// $license_data->license will be either "deactivated" or "failed"
		if( $license_data->license == 'deactivated' )
			delete_option('wpsl_license_status');

	}
}

function edd_register_option() {
	register_setting('wpsl_license', 'wpsl_license_key', array(__CLASS__, 'edd_sanitize_license'));
}

  function edd_activate_license() {
  // listen for our activate button to be clicked
  if( isset( $_POST['_edd_license_activate'] ) ) {
    // run a quick security check
  //   if( ! check_admin_referer( 'edd_nonce', 'edd_nonce' ) ) 	
	//		return; // get out if we didn't click the Activate button
			

    // retrieve the license from the database
    $license = trim( get_option( 'wpsl_license_key' ) );


    // data to send in our API request
    $api_params = array(
      'edd_action'=> 'activate_license',
      'license'   => $license,
      'item_name' => urlencode( WPSL_ITEM_NAME ) // the name of our product in EDD
    );

    // Call the custom API.
    $response = wp_remote_get( add_query_arg( $api_params, WPSL_STORE_URL ), array( 'timeout' => 15, 'sslverify' => false ) );

    // make sure the response came back okay
    if ( is_wp_error( $response ) )
      return false;

    // decode the license data
    $license_data = json_decode( wp_remote_retrieve_body( $response ) );

    // $license_data->license will be either "active" or "inactive"

    update_option( 'wpsl_license_status', $license_data->license );

  }
  }

  function activate() {
    if (!get_option(WPSL_INSTALL_TIME)) {
      update_option(WPSL_INSTALL_TIME, time());
    }
  }

  function license_page() {
  $license = get_option('wpsl_license_key');
  $status = get_option('wpsl_license_status');

  //var_dump($license, $status);

  if (get_option('wpsl_license_status') != 'valid') {
    $activate = get_option(WPSL_INSTALL_TIME);
    if ($activate + 86400*3 > time()) {
      echo '<h3>Trial Period</h3>';
      echo '<p>Your 3 day trial period will end on ' . date(get_option('date_format') . ' @ ' . get_option('time_format'), $activate + 86400*3)  . '</p>';
    } else {
      echo '<p>Your 3 day trial period has ended. Please purchase a license.</p>';
    }
    echo '<a href="http://www.wordpresssecuritylab.com/license" target="_blank" class="button">Buy a license now</a>';
    echo '<br><br>';
  }

  ?>
    <h3>License Details</h3>
    <form method="post" action="options.php">

      <?php settings_fields('wpsl_license');  ?>

      <table class="form-table">
        <tbody>
          <tr valign="top">
            <th scope="row" valign="top"><label class="description" for="wpsl_license_key">License Key</label></th>
            <td>
              <input id="wpsl_license_key" name="wpsl_license_key" type="text" class="regular-text" value="<?php esc_attr_e( $license ); ?>" />
            </td>
          </tr>
          <?php if( false !== $license ) { ?>
            <tr valign="top">
              <th scope="row" valign="top">License Status</th>
              <td>
                <?php if( $status !== false && $status == 'valid' ) { ?>
                  <strong>active &amp; valid </strong>
                  <input type="submit" class="button-secondary" name="_edd_license_deactivate" value="Deactivate License" />
                <?php } else {
                   ?>
                  <strong>inactive </strong>
                  <input type="submit" class="button-secondary" name="_edd_license_activate" value="Activate License" />
                <?php } ?>
              </td>
            </tr>
          <?php } ?>
        </tbody>
      </table>
      <?php submit_button('Save License Key'); ?>
    </form>
  <?php
  }

  function license() {
    if (get_option('wpsl_license_status') == 'valid') {
      return true;
    }

    $activate = get_option(WPSL_INSTALL_TIME);
    if ($activate && ($activate + 86400*3 > time())) {
      return true;
    }

    return false;
  }
} // wpsl class


// hook everything up
add_action('init', array('wpsl', 'init'));
add_action('admin_init', array('wpsl', 'edd_register_option'));
add_action('init', array('wpsl_cs', 'init'));
register_activation_hook(__FILE__, array('wpsl', 'activate'));

// when deativated clean up
register_deactivation_hook(__FILE__, array('wpsl', 'deactivate'));
?>